@@ -13,7 +13,7 @@ from couchdbkit.resource import ResourceNotFound
 import logging
 import hashlib
 from copy import copy
-from couchforms.signals import xform_archived
+from couchforms.signals import xform_archived, xform_unarchived
 from dimagi.utils.mixins import UnicodeMixIn
 from couchforms.const import ATTACHMENT_NAME
 from couchdbkit.exceptions import PreconditionFailed
@@ -69,11 +69,22 @@ class Metadata(DocumentSchema):
     deprecatedID = StringProperty()
     username = StringProperty()
 
+class XFormOperation(DocumentSchema):
+    """
+    Simple structure to represent something happening to a form.
+
+    Currently used just by the archive workflow.
+    """
+    user = StringProperty()
+    date = DateTimeProperty(default=datetime.datetime.utcnow)
+    operation = StringProperty() # e.g. "archived", "unarchived"
+
 class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
     """An XForms instance."""
     xmlns = StringProperty()
     received_on = DateTimeProperty()
     partial_submission = BooleanProperty(default=False) # Used to tag forms that were forcefully submitted without a touchforms session completing normally
+    history = SchemaListProperty(XFormOperation)
     
     @property
     def get_form(self):
@@ -96,7 +107,7 @@ class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
     @property
     def _form(self):
         return self[const.TAG_FORM]
-    
+
     @property
     def type(self):
         return self._form.get(const.TAG_TYPE, "")
@@ -187,7 +198,10 @@ class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
             return self.fetch_attachment(ATTACHMENT_NAME)
         except ResourceNotFound:
             logging.warn("no xml found for %s, trying old attachment scheme." % self.get_id)
-            return self[const.TAG_XML]
+            try:
+                return self[const.TAG_XML]
+            except AttributeError:
+                return None
     
     @property
     def attachments(self):
@@ -206,9 +220,18 @@ class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
         order they are found.
         
         """
-        xml_payload = self.get_xml()
-        element = ElementTree.XML(xml_payload)
         to_return = SortedDict()
+
+        xml_payload = self.get_xml()
+        if not xml_payload:
+            return SortedDict(sorted(self.form.items()))
+
+        try:
+            element = ElementTree.XML(xml_payload)
+        except UnicodeEncodeError:
+            xml_payload = xml_payload.encode('utf-8', errors='replace')
+            element = ElementTree.XML(xml_payload)
+
         for child in element:
             # fix {namespace}tag format forced by ElementTree in certain cases (eg, <reg> instead of <n0:reg>)
             key = child.tag.split('}')[1] if child.tag.startswith("{") else child.tag 
@@ -217,11 +240,24 @@ class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
             to_return[key] = self.xpath('form/' + key)
         return to_return
 
-    def archive(self):
+    def archive(self, user=None):
         self.doc_type = "XFormArchived"
+        self.history.append(XFormOperation(
+            user=user,
+            operation='archive',
+        ))
         self.save()
         xform_archived.send(sender="couchforms", xform=self)
 
+    def unarchive(self, user=None):
+        self.doc_type = "XFormInstance"
+        self.history.append(XFormOperation(
+            user=user,
+            operation='unarchive',
+        ))
+        XFormInstance.save(self) # subclasses explicitly set the doc type so force regular save
+        xform_unarchived.send(sender="couchforms", xform=self)
+
 class XFormError(XFormInstance):
     """
     Instances that have errors go here.
@@ -264,7 +300,6 @@ class XFormArchived(XFormError):
     """
     Archived forms don't show up in reports
     """
-    archived_date = DateTimeProperty(default=datetime.datetime.utcnow)
 
     def save(self, *args, **kwargs):
         # force set the doc type and call the right superclass
@@ -306,4 +341,9 @@ class SubmissionErrorLog(XFormError):
         error.put_attachment(instance, ATTACHMENT_NAME)
         error.save()
         return error
-    
+
+
+class DefaultAuthContext(DocumentSchema):
+
+    def is_valid(self):
+        return True
